from django import forms
from .models import AbstractSolarluxUser


class UserAddWithoutPasswordForm(forms.ModelForm):
    class Meta:
        model = AbstractSolarluxUser
        fields = ('username',)

    def save(self, commit=True):
        user = super().save(commit=False)
        user.set_unusable_password()
        if commit:
            user.save()
        return user



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-11-07 15:28
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("example", "0004_auto_20171101_1816")]

    operations = [
        migrations.AlterField(
            model_name="myuser",
            name="customer_category",
            field=models.CharField(
                blank=True,
                choices=[
                    ("ACTIVE_PARTNER", "active partner"),
                    ("BI_FOLDING_DOOR_SPECIALIST", "bi-folding-door specialist"),
                    ("DEALER", "dealer"),
                    ("ARCHITECT_PLANNER", "architect/planner"),
                    ("HOUSING_SOCIETY", "housing society"),
                    ("PRIVATE_CUSTOMER", "private customer"),
                    ("OTHERS", "others"),
                ],
                max_length=200,
                null=True,
                verbose_name="customer category",
            ),
        ),
        migrations.AlterField(
            model_name="myuser",
            name="customer_name",
            field=models.CharField(
                blank=True, max_length=200, null=True, verbose_name="customer name"
            ),
        ),
        migrations.AlterField(
            model_name="myuser",
            name="first_name",
            field=models.CharField(
                blank=True, max_length=30, null=True, verbose_name="first name"
            ),
        ),
        migrations.AlterField(
            model_name="myuser",
            name="last_name",
            field=models.CharField(
                blank=True, max_length=150, null=True, verbose_name="last name"
            ),
        ),
        migrations.AlterField(
            model_name="myuser",
            name="title",
            field=models.CharField(
                blank=True, max_length=20, null=True, verbose_name="title"
            ),
        ),
    ]



from django.conf.urls import url

from slxauth.views import LoginView, CallbackView, LogoutView, TestConnectionView

""" Include in urlpatterns like this:
url(r'^accounts/', include('slxauth.urls', namespace='slxauth'))
"""

app_name = "slxauth"

urlpatterns = [
    url(r"^login/$", LoginView.as_view(), name="login"),
    url(r"^logout/$", LogoutView.as_view(), name="logout"),
    url(r"^callback/$", CallbackView.as_view(), name="callback"),
    url(r"^ping/$", TestConnectionView.as_view(), name="test_connection")
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-22 12:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("example", "0006_auto_20171110_1501")]

    operations = [
        migrations.AddField(
            model_name="myuser",
            name="access_token",
            field=models.TextField(
                blank=True, null=True, verbose_name="OAuth2 Access Token"
            ),
        )
    ]



# Generated by Django 2.0.1 on 2018-02-08 09:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("example", "0007_myuser_access_token")]

    operations = [
        migrations.RemoveField(model_name="myuser", name="avatar_attachment_id"),
        migrations.RemoveField(model_name="myuser", name="company_logo_attachment_id"),
        migrations.AlterField(
            model_name="myuser",
            name="customer_category",
            field=models.CharField(
                blank=True,
                choices=[
                    ("QUALITY_PARTNER", "quality partner"),
                    ("DEALER", "dealer"),
                    ("ARCHITECT_PLANNER", "architect/planner"),
                    ("HOUSING_SOCIETY", "housing society"),
                    ("PRIVATE_CUSTOMER", "private customer"),
                    ("OTHERS", "others"),
                    ("INTERNAL", "internal"),
                ],
                max_length=200,
                null=True,
                verbose_name="customer category",
            ),
        ),
        migrations.AlterField(
            model_name="myuser",
            name="customer_no",
            field=models.CharField(
                blank=True, max_length=255, null=True, verbose_name="customer no"
            ),
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.rst")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="django-slxauth",
    version="1.14.1",
    packages=find_packages("."),
    include_package_data=True,
    license="BSD License",
    description="Authentication provider for Solarlux Auth Service.",
    long_description=README,
    url="https://git.solarlux.com/sl-it-dev/slx8124_django-slxauth/",
    author="Enno Lohmeier",
    author_email="e.lohmeier@solarlux.de",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    keywords="django auth account oauth",
    install_requires=["Django", "cryptography", "PyJWT", "requests"],
)



import logging

import jwt
import requests
from django.conf import settings
from django.contrib.auth import get_user_model, login
from django.contrib.auth.models import Group
from jwt import InvalidTokenError

logger = logging.getLogger(__name__)


def is_authorized(authorities, required):
    """
    :param authorities: list of given roles
    :param required: list of required roles
    :returns: true if any required role exists
    """
    return len(set(required) & set(authorities)) > 0


def get_authenticator():
    return TokenAuthenticator(
        settings.OAUTH_ROLES_REQUIRED,
        settings.OAUTH_ROLES_STAFF,
        settings.OAUTH_ROLES_SUPERUSER,
        settings.OAUTH_PUBLIC_KEY,
    )


class TokenAuthenticator:
    def __init__(self, required_roles, staff_roles, superuser_roles, oauth_publickey):
        self.required_roles = required_roles
        self.staff_roles = staff_roles
        self.superuser_roles = superuser_roles
        self.oauth_publickey = oauth_publickey

    def _getpk(self, minipk):
        """
        ### mySolarlux convention ###
        since the publickey is stored in a no-newline env-var we need to add a prefix and suffix to it,
        to be able to use it as a jwt publickey signature
        """
        return (
            """-----BEGIN PUBLIC KEY-----
%s
-----END PUBLIC KEY-----"""
            % minipk
        )

    def decode_token(self, token):
        oauth_public_key = self._getpk(self.oauth_publickey)
        # validate the oauth2 token
        return jwt.decode(token, oauth_public_key, algorithms=["RS256"])

    def check_access(self, authorities):
        # check if superuser
        if is_authorized(authorities, self.superuser_roles):
            return True

        # if not superuser, check if any required role exists
        return len(self.required_roles) == 0 or is_authorized(
            authorities, self.required_roles
        )

    def get_unsaved_user_from_token(self, access_token, token_data):

        if not self.check_access(token_data["authorities"]):
            return None

        user_model = get_user_model()
        user, created = user_model.objects.get_or_create(
            username="um_%s" % token_data["id"]
        )

        logger.debug("%s user %s" % ("creating" if created else "updating", user.email))

        user.access_token = access_token
        user.customer_no = token_data["customerNumber"]
        user.customer_name = token_data["customerName"]
        user.last_name = token_data["lastName"]
        user.first_name = token_data["firstName"]
        user.title = token_data["title"]
        user.is_customer_admin = token_data["isCustomerAdmin"]
        user.language = token_data["language"]
        user.crm_contact_id = token_data["crmContactId"]
        user.email = token_data["email"]
        user.sap_pernr = token_data["sapPernr"] if "sapPernr" in token_data else None
        user.sap_sales_org = token_data["sapSalesOrg"] if "sapSalesOrg" in token_data else None
        user.sap_company_code = token_data["sapCompanyCode"] if "sapCompanyCode" in token_data else None
        user.country_code = token_data["countryCode"] if "countryCode" in token_data else None
        user.department = token_data["department"] if "department" in token_data else None
        user.mobile_number = token_data["mobileNumber"] if "mobileNumber" in token_data else None

        user_groups = user.groups.all()

        logger.debug("deleting exited groups for user %s" % user.email)
        for group in user_groups:
            if group.name not in token_data["authorities"]:
                group.user_set.remove(user)

        logger.debug("creating joined groups for user %s" % user.email)
        for role in token_data["authorities"]:
            if role not in [x.name for x in user_groups]:
                group, created = Group.objects.get_or_create(name=role)
                user.groups.add(group)

        user.is_active = self.check_access(token_data["authorities"])
        user.is_staff = is_authorized(token_data["authorities"], self.staff_roles)
        user.is_superuser = is_authorized(
            token_data["authorities"], self.superuser_roles
        )

        logger.debug("finished user extraction from token")
        return user

    def login_using_token(self, request, access_token):

        logger.debug("extracting user info from access-token %s" % access_token)
        try:
            # validate the oauth2 token
            token_data = self.decode_token(access_token)
        except InvalidTokenError:
            logger.info("token invalid", exc_info=True)
            return False

        if not self.check_access(token_data["authorities"]):
            return False

        logger.debug("getting user from access token without portal config")
        user = self.get_unsaved_user_from_token(access_token, token_data)

        logger.debug("saving user object")
        user.save()
        logger.debug("logging in user %s" % user.email)
        login(request, user)
        return True



"""
Django settings for web project.

Generated by 'django-admin startproject' using Django 1.11.3.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""

import os
from django.utils.translation import ugettext_lazy as _

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "@@n624%u@h@jcfrjkl9_me_#_k$a^e9w@=pjcui#z--djvqh9n"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]


# Application definition

INSTALLED_APPS = [
    "example.apps.ExampleConfig",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

# this is the portal domain
PORTAL_DOMAIN = os.environ.get("DOMAIN", "my-dev.solarlux.com")

OAUTH_ENABLED = bool(os.environ.get("OAUTH_ENABLED", False))
OAUTH_URL = os.environ.get("OAUTH_URL", "https://%s/auth/oauth" % PORTAL_DOMAIN)
OAUTH_COOKIE_DOMAIN = os.environ.get("OAUTH_COOKIE_DOMAIN", "")
OAUTH_CLIENT_ID = os.environ.get(
    "OAUTH_CLIENT_ID", "localdev1"
)
OAUTH_CLIENT_SECRET = os.environ.get("OAUTH_CLIENT_SECRET", "6Htrqkzel6uqeuHZZrjA5Q2EXka4kw10")
OAUTH_PUBLIC_KEY = os.environ.get(
    "OAUTH_PUBLIC_KEY",
    "MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAiahLhO5+1bWfqSQhdCR7RJLpveU0OICX5rPb4N5AMx6uGhwCgO9U44kkg1Z/K+atcokYJUW72pij0TjFpz9XOAIR1BbpzpsSE/ixe1cRx2kDG1ABiCvWBsldnuuW/KhZi9FtQGnVBysKiXT/0Y87MtJo63ZQwKOK9DSdQWiz/zWgij9oR4Q2Cl3iBrQ3h8HMYEswhKrZfO+GEz8BKUgLeUMr6TJtJQX85Z+ulWcTsxbAZZPzAiNlnBbbd8MV3rimEcfE/0vbRoFXZ/yfQ8PlD09MdrCXjCGnr1WKrKkse9HgTDjnBiewMZiHswYfHHiEqSCw4o3U2XtjGGXWSZiv5wIDAQAB",
)
OAUTH_ROLES_REQUIRED = os.environ.get("OAUTH_ROLES_REQUIRED", "").split()
OAUTH_ROLES_STAFF = os.environ.get("OAUTH_ROLES_STAFF", "ROLE_SOLARLUX_ADMIN").split()
OAUTH_ROLES_SUPERUSER = os.environ.get(
    "OAUTH_ROLES_SUPERUSER", "ROLE_SOLARLUX_ADMIN"
).split()

if OAUTH_ENABLED:
    MIDDLEWARE += ["slxauth.middleware.TokenAuthMiddleware"]


# Logging
# https://docs.djangoproject.com/en/1.11/topics/logging/

LOGGING = {
    "version": 1,
    "disable_existing_loggers": True,
    "formatters": {
        "standard": {"format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s"}
    },
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "standard",
        }
    },
    "loggers": {
        "django": {"handlers": ["console"], "level": "INFO", "propagate": True},
        "slxauth": {"handlers": ["console"], "level": "INFO", "propagate": True},
        "portal_ui": {"handlers": ["console"], "level": "INFO", "propagate": True},
        "app": {"handlers": ["console"], "level": "INFO", "propagate": True},
    },
}

if DEBUG:
    LOGGING["loggers"]["django"]["level"] = "DEBUG"
    LOGGING["loggers"]["slxauth"]["level"] = "DEBUG"
    LOGGING["loggers"]["portal_ui"]["level"] = "DEBUG"
    LOGGING["loggers"]["app"]["level"] = "DEBUG"

LOGSTASH_ENABLED = bool(os.environ.get("LOGSTASH_ENABLED", False))
LOGSTASH_HOST = os.environ.get("LOGSTASH_HOST", "srv-appmetrics.slx.de")
LOGSTASH_PORT = int(os.environ.get("LOGSTASH_PORT", 5959))
LOGSTASH_TAGS = os.environ.get("LOGSTASH_TAGS", "").split()
if LOGSTASH_ENABLED:
    LOGGING["handlers"]["logstash"] = {
        "level": "INFO",
        "class": "logstash.LogstashHandler",
        "host": LOGSTASH_HOST,
        "port": LOGSTASH_PORT,
        "version": 1,
        "message_type": "logstash",
        "fqdn": False,
        "tags": LOGSTASH_TAGS,
    }
    LOGGING["loggers"]["django"]["handlers"].append("logstash")
    LOGGING["loggers"]["slxauth"]["handlers"].append("logstash")
    LOGGING["loggers"]["portal_ui"]["handlers"].append("logstash")
    LOGGING["loggers"]["app"]["handlers"].append("logstash")

ROOT_URLCONF = "web.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

WSGI_APPLICATION = "web.wsgi.application"


# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]


# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = "en-gb"
TIME_ZONE = "Europe/Berlin"
USE_I18N = True
USE_L10N = True
USE_TZ = True

LANGUAGES = [
    ("de", _("German")),
    ("en-gb", _("British English")),
    ("nl", _("Dutch")),
    ("fr", _("French")),
]

LOCALE_PATHS = (os.path.join(BASE_DIR, "locale"),)


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

SCRIPT_NAME = os.environ.get("SCRIPT_NAME", "")
LOGIN_URL = "%s/accounts/login" % SCRIPT_NAME

STATIC_URL = "%s/static/" % SCRIPT_NAME
MEDIA_URL = "%s/media/" % SCRIPT_NAME

LOGOUT_REDIRECT_URL = "%s/" % SCRIPT_NAME

AUTH_USER_MODEL = "example.MyUser"
